﻿/*
 * Created by SharpDevelop.
 * User: johann
 * Date: 02/03/2012
 * Time: 17:56
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using SpringCardPCSC;
using SpringCardUtils;

namespace SpringCardApplication
{
	public partial class MainForm : Form
	{
		ShadowForm shade;
		SCardReader reader = null;
		NfcEmulDesfireEV0 picc = null;
		int nfc_status = 0;
		
		public MainForm()
		{
		  Settings s = new Settings();
		  if (s.ShowConsole)
		    SpringCardUtils.SystemConsole.Show();
			InitializeComponent();		
			ShowStatus();
		}
		
		void ShowStatus()
		{
		  string s;
		  int i;
		  
		  switch (nfc_status)
		  {
		    case 1 :
		      s = "Desfire emulation: active";
		      i = 10;
		      break;
		    case 0 :
		      s = "Desfire emulation: stopped";
		      i = 1;
		      break;
		    default :
		      s = "Desfire emulation: unknown?";
		      i = 6;
		      break;
		  }

		  miNfcStop.Enabled = (nfc_status == 1);
		  miNfcStart.Enabled = (nfc_status != 1);
		  btnNfcControl.Text = s;
		  btnNfcControl.ImageIndex = i;
		}
			
		void SetNfcMode(bool active, bool silent)
		{
      nfc_status = -1;
      
      ePiccStatus.Text = "";
      ePiccCommand.Text = "";
      ePiccResponse.Text = "";

      if (active)
		  {        
  		  picc = new NfcEmulDesfireEV0();
  		  Settings s = new Settings();
  		  if (s.ShowConsole)
  		    picc.Trace = true;  		  
  		  if (s.ShowConsole && s.DetailedConsole)
  		    picc.Debug = true;  		  
  		  picc.OnError = PiccError;
  		  picc.OnSelected = PiccSelected;
  		  picc.OnDeselected = PiccDeselected;
  		  picc.OnCommandReceived = PiccCommandReceived;
  		  picc.OnResponseReady = PiccResponseReady;
  		  if (picc.Start(reader.Name))
  		    nfc_status = 1;
		  } else
      if (picc != null)
      {
        picc.Stop();
        picc = null;
        nfc_status = 0;
      }

		  ShowStatus();
		}
		
		delegate void PiccErrorInvoker();
		void PiccError()
		{
		  /* See comment in ReaderStatusChange for explanation! */
			if (InvokeRequired)
			{
				this.BeginInvoke(new PiccErrorInvoker(PiccError));
				return;				
			}
      ePiccStatus.Text = "Fatal error, emulation aborted";      
      if (picc != null)
      {
        picc.Stop();
        picc = null;
      }
      nfc_status = -1;
      ShowStatus();
		}

		delegate void PiccSelectedInvoker();
		void PiccSelected()
		{
			if (InvokeRequired)
			{
				this.BeginInvoke(new PiccSelectedInvoker(PiccSelected));
				return;				
			}		  
      ePiccStatus.Text = "Selected";
		}

		delegate void PiccDeselectedInvoker();
		void PiccDeselected()
		{
			if (InvokeRequired)
			{
				this.BeginInvoke(new PiccDeselectedInvoker(PiccDeselected));
				return;				
			}		  
      ePiccStatus.Text = "Idle";
		}

		delegate void PiccCommandReceivedInvoker(CardBuffer Command);
		void PiccCommandReceived(CardBuffer Command)
		{
		  /* See comment in ReaderStatusChange for explanation! */
			if (InvokeRequired)
			{
				this.BeginInvoke(new PiccCommandReceivedInvoker(PiccCommandReceived), Command);
				return;				
			}			
			ePiccCommand.Text = Command.AsString(" ");
			ePiccResponse.Text = "";
		}

		delegate void PiccResponseReadyInvoker(CardBuffer Response);
		void PiccResponseReady(CardBuffer Response)
		{
		  /* See comment in ReaderStatusChange for explanation! */
			if (InvokeRequired)
			{
				this.BeginInvoke(new PiccResponseReadyInvoker(PiccResponseReady), Response);
				return;				
			}
			ePiccResponse.Text = Response.AsString(" ");
		}
		
	

		delegate void ReaderStatusChangeInvoker(uint ReaderState, CardBuffer CardAtr);		
		void ReaderStatusChange(uint ReaderState, CardBuffer CardAtr)
		{
		  /* The ReaderStatusChange function is called as a delegate (callback) by the SCardReader object     */
		  /* within its backgroung thread. Therefore we must use the BeginInvoke syntax to switch back from   */
		  /* the context of the background thread to the context of the application's main thread. Overwise   */
		  /* we'll get a security violation when trying to access the window's visual components (that belong */ 
		  /* to the application's main thread and can't be safely manipulated by background threads).         */
			if (InvokeRequired)
			{
				this.BeginInvoke(new ReaderStatusChangeInvoker(ReaderStatusChange), ReaderState, CardAtr);
				return;				
			}			
		
			eReaderStatus.Text = SCARD.ReaderStatusToString(ReaderState);
			
			if (CardAtr != null)
			{
			  eCardAtr.Text = CardAtr.AsString(" ");
			} else
			{
			  eCardAtr.Text = "";
			}
			
			if (ReaderState == SCARD.STATE_UNAWARE)
			{
//			  lbReaderStatus.Text = "";
			  nfc_status = -1;
			  ShowStatus();
			  Application.Exit();

			  MessageBox.Show("The reader we were working on has disappeared from the system. This application will terminate now.",
			                  "The reader has been removed",
			                  MessageBoxButtons.OK,
			                  MessageBoxIcon.Exclamation);

			  
			} else
  	  if ((ReaderState & SCARD.STATE_EMPTY) != 0)
			{
//			  lbReaderStatus.Text = "No card in the reader";
			} else
 	  if ((ReaderState & SCARD.STATE_UNAVAILABLE) != 0)
			{
//			  lbReaderStatus.Text = "Reader in use";
			} else			  
			if ((ReaderState & SCARD.STATE_MUTE) != 0)
			{
//			  lbReaderStatus.Text = "Card is mute";
			} else
	    if ((ReaderState & SCARD.STATE_INUSE) != 0)
			{
//			  lbReaderStatus.Text = "Card in use";
			} else
	    if ((ReaderState & SCARD.STATE_PRESENT) != 0)
			{
			  //lbReaderStatus.Text = "Card ready";
			}
						
			ShowStatus();
		}
		

		void MainFormShown(object sender, EventArgs e)
		{
			SplashForm splash = new SplashForm();
			splash.ShowDialog();
			
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			
			if (readerSelect.SelectedReader == null)
			{
			  readerSelect.Preselect("SpringCard|NFC|tactless");
			  for (;;)
			  {
			    readerSelect.ShowDialog();
			    if (readerSelect.SelectedReader != null)
			      break;
			    
			    if (MessageBox.Show("This application can't run without a reader. Do you want to leave the application now ?",
			                        "No reader selected",
			                        MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
			    {
			      Application.Exit();
			      return;
			    }			    
			  }			  
			}
			
			eReaderName.Text = readerSelect.SelectedReader;
			eReaderStatus.Text = "";
			eCardAtr.Text = "";
					
			shade.Dispose();
			shade = null;
			
	    reader = new SCardReader(eReaderName.Text);
	    reader.StartMonitor(new SCardReader.StatusChangeCallback(ReaderStatusChange));
		}
		
		void MainFormLoad(object sender, EventArgs e)
		{
		  shade = new ShadowForm();
			shade.ShowShadow(this);
		}
		
		void AboutToolStripMenuItemClick(object sender, EventArgs e)
		{
		  shade = new ShadowForm();
			shade.ShowShadow(this);
			AboutForm f = new AboutForm();
			f.ShowDialog();
			shade.Dispose();
			shade = null;
		}
		
		void MainFormFormClosed(object sender, FormClosedEventArgs e)
		{
		  if (reader != null)
		  {
		    reader.StopMonitor();
		    reader = null;
		  }
		}
		
		void ChangeReader()
		{
		  if (reader != null)
		  {
		    reader.StopMonitor();
		    reader = null;
		  }
		  
	    nfc_status = -1;
	    ShowStatus();
	    eReaderStatus.Text = "";
	    eCardAtr.Text = "";

		  shade = new ShadowForm();
		  shade.ShowShadow(this);
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			readerSelect.SelectedReader = eReaderName.Text;
      readerSelect.ShowDialog();
	    if (readerSelect.SelectedReader != null)
			  eReaderName.Text = readerSelect.SelectedReader;
	    shade.Dispose();
	    shade = null;
	    
	    reader = new SCardReader(eReaderName.Text);
			SetNfcMode(false, false);
	    reader.StartMonitor(new SCardReader.StatusChangeCallback(ReaderStatusChange));
		}
			
		void MainFormFormClosing(object sender, FormClosingEventArgs e)
		{
		  if (reader != null)
		  {
        if (picc != null)
        {
          picc.Stop();
          picc = null;
        }
		  }
		}
		
		void imgLogoClick(object sender, EventArgs e)
		{
			System.Diagnostics.Process.Start("http://www.springcard.com");		  
		}
		
		void BtnNfcControlClick(object sender, EventArgs e)
		{
		  switch (nfc_status)
		  {
		    case 1 :
		      SetNfcMode(false, true);
		      break;
		    default :
		      SetNfcMode(true, true);
		      break;
		  }
		}		

		void BtnReaderChangeClick(object sender, System.EventArgs e)
		{
		  ChangeReader();		  
		}
		
		void QuitToolStripMenuItemClick(object sender, EventArgs e)
		{
		  Close();
		}
		
		
		void miNFCStartClick(object sender, EventArgs e)
		{
		  SetNfcMode(true, true);
		}
		
		void StopNFCEmulationToolStripMenuItemClick(object sender, EventArgs e)
		{
		  SetNfcMode(false, true);
		}
		
	
		void MiSettingsClick(object sender, EventArgs e)
		{
		  shade = new ShadowForm();
			shade.ShowShadow(this);
			SettingsForm f = new SettingsForm();
			f.ShowDialog();
			shade.Dispose();
			shade = null;
		  Settings s = new Settings();
		  if (s.ShowConsole)
		    SpringCardUtils.SystemConsole.Show();
		  else
		    SpringCardUtils.SystemConsole.Hide();
		}
	}
}
