import java.io.*;
import java.util.List;
import java.util.ArrayList;
import javax.smartcardio.*;

public class cardemul
{
  public static void main(String[]args)
  {    
    String uri = "http://www.springcard.com";
    byte[] ndef;
    
    ArrayList<CardTerminal> terminalsWithEmulEnabled = new ArrayList<CardTerminal>();
  
    try
    {
      /* Get the list of Card Terminals */
      /* ------------------------------ */
      
      TerminalFactory factory = TerminalFactory.getDefault();
      CardTerminals terminalList = factory.terminals();

      /* Try to put every Terminal into Card Emulation mode (NFC Forum Type 4 Tag) */
      /* ------------------------------------------------------------------------- */
      
      for (int i = 0; i < terminalList.list().size(); i++)
      {
        CardTerminal terminal = terminalList.list().get(i);
      
        System.out.println("Reader " + i + ": " + terminal.getName());
        System.out.println("\tCard State : ");
        String cardStatus = " No card in the reader";

        if (terminal.isCardPresent())
          cardStatus = " Card Present";
        System.out.println("\t\t" + cardStatus);
        
        if (startNfcTagType4Emulation(terminal))
        {
          System.out.println("\t\tCard emulation activated");
          terminalsWithEmulEnabled.add(terminal);
        }
      }
    }
    catch (CardException e)
    {
      System.out.println("List Card Terminals: Exception");
      System.out.println("\t" + e.getMessage());
      return;
    }          
      
    /* Write a valid NDEF into every emulated Tag */
    /* ------------------------------------------ */
    
    try
    {
      ndef = serializeNdefRecord(
        NDEF_TNF_NFC_RTD_WKN, /* TNF = well-known */
        "U".getBytes("US-ASCII"), /* Type = "U" (URI) */
        null, /* No ID */
        serializeUriPayload(uri) /* Compressed URI */
      );
      
      /* Single NDEF */
      ndef[0] |= NDEF_HEADER_MESSAGE_BEGIN | NDEF_HEADER_MESSAGE_END;
    }
    catch (UnsupportedEncodingException e)
    {
      System.out.println("Unsupported encoding");
      return;      
    }
    catch (NotSerializableException e)
    {
      System.out.println("Can't serialize the NDEF");
      return;          
    }   
    
    for (int i = 0; i < terminalsWithEmulEnabled.size(); i++)
    {
      try
      {
        CardTerminal terminal = terminalsWithEmulEnabled.get(i);
        Card card = terminal.connect("*");
        writeNdefOnType4(card.getBasicChannel(), ndef);
        card.disconnect(false);
      }
      catch (CardException e)
      {
        System.out.println("Write NDEF on emulated Tags: Exception");
        System.out.println("\t" + e.getMessage());
        return;      
      }
    }
    
    /* Wait....................... */
    /* --------------------------- */
      
    System.out.println("Hit <Enter> to leave the Card Emulation mode");    
    try
    {
      System.in.read();
    }
    catch (IOException e)
    {
    
    }

    /* Put the Terminals in normal (reader) mode again when leaving */
    /* ------------------------------------------------------------ */
    
    for (int i = 0; i < terminalsWithEmulEnabled.size(); i++)
    {
      backToReaderMode(terminalsWithEmulEnabled.get(i));
    }      
  }
  
  public static final byte NDEF_TNF_EMPTY                = (byte) 0x00;
  public static final byte NDEF_TNF_NFC_RTD_WKN          = (byte) 0x01;
  public static final byte NDEF_TNF_MEDIA_TYPE           = (byte) 0x02;
  public static final byte NDEF_TNF_ABSOLUTE_URI         = (byte) 0x03;
  public static final byte NDEF_TNF_NFC_RTD_EXT          = (byte) 0x04;
  public static final byte NDEF_TNF_UNKNOWN              = (byte) 0x05;
  public static final byte NDEF_TNF_UNCHANGED            = (byte) 0x06;
  public static final byte NDEF_TNF_RESERVED             = (byte) 0x07;

  public static final byte NDEF_HEADER_MESSAGE_BEGIN     = (byte) 0x80;
  public static final byte NDEF_HEADER_MESSAGE_END       = (byte) 0x40;

  public static final byte NDEF_HEADER_CHUNK_FLAG        = (byte) 0x20;
  public static final byte NDEF_HEADER_SHORT_RECORD      = (byte) 0x10;
  public static final byte NDEF_HEADER_ID_LENGTH_PRESENT = (byte) 0x08;

  public static final byte NDEF_HEADER_RECORD_TYPE_MASK  = (byte) 0x07;
  
  public static final byte NDEF_FILE_CONTROL_TLV         = (byte) 0x04;
  public static final byte NDEF_HEADER_MAGIC_VALUE       = (byte) 0xD1;
  
  private static final byte[] NDEF_APPLICATION_ID        = { (byte) 0xD2, (byte) 0x76, (byte) 0x00, (byte) 0x00, (byte) 0x85, (byte) 0x01, (byte) 0x01 };
  private static final byte[] NDEF_CC_FILE_ID            = { (byte) 0xE1, (byte) 0x03 };
  
  /*
   * serializeUriPayload
   * -------------------
   */
  public static byte[] serializeUriPayload(String uri) throws UnsupportedEncodingException
  {
    byte[] b = uri.getBytes("US-ASCII");
    byte[] r;
    
    if (uri.startsWith("http://www."))
    {
      r = new byte[b.length - 10];
      r[0] = 0x01;
      for (int i=1; i<r.length; i++)
        r[i] = b[i+10];
    } else
    {
      r = new byte[b.length + 1];
      r[0] = 0x00;
      for (int i=1; i<r.length; i++)
        r[i] = b[i-1];    
    }
        
    return r;
  }  
  
  /*
   * serializeNdefRecord
   * -------------------
   * Transform NDEF data into a byte[] that could be written onto a Tag
   */  
  public static byte[] serializeNdefRecord(byte tnf, byte[] type, byte[] id, byte[] payload) throws NotSerializableException
  {
    int o, i;
    byte header = tnf;
    int type_length = 0;
    int id_length = 0;
    int payload_length = 0;
    
    /* Prepare */
    /* ------- */
    
    o = 1;
    
    /* Type length on 1 byte */
    type_length = 0;    
    if (type != null)
      type_length = type.length;
    if (type_length > 254)
      throw new NotSerializableException();

    o += 1;
    o += type_length;

    /* Content length on 1 or 4 bytes */
    if (payload != null)
    {
      payload_length = payload.length;
      if (payload_length <= 254)
      {
        o += 1;
        header |= NDEF_HEADER_SHORT_RECORD;
      } else
      {
        o += 4;
      } 
      o += payload_length;
    }
    
    /* ID length if ID is present */
    if (id != null)
      id_length = id.length;
    if (id_length > 254)
      throw new NotSerializableException();
    
    if (id_length != 0)
    {
      o += 1;
      o += id_length;
      header |= NDEF_HEADER_ID_LENGTH_PRESENT;
    }
    
    /* Execute */
    /* ------- */
    
    byte[] dst = new byte[o];
    
    o = 0;
    
    /* Header */  
    dst[o++] = header;
    
    /* Type length on 1 byte */    
    dst[o++] = (byte) type_length;
    
    /* Payload length on 1 or 4 bytes */
    if ((header & NDEF_HEADER_SHORT_RECORD) != 0)
    {
      dst[o++] = (byte) payload_length;
    } else
    {
      dst[o++] = 0;
      dst[o++] = 0;
      dst[o++] = (byte) (payload_length / 0x0100);
      dst[o++] = (byte) (payload_length % 0x0100);
    }
    
    /* ID length */
    if ((header & NDEF_HEADER_ID_LENGTH_PRESENT) != 0)
    {
      dst[o++] = (byte) (id_length);
    }
    
    /* Type */
    if (type_length != 0)
    {
      for (i=0; i<type_length; i++)
        dst[o++] = type[i];
    }
      
    /* ID */
    if (id_length != 0)
    {
      for (i=0; i<id_length; i++)
        dst[o++] = id[i];
    }

    /* Content */
    if (payload_length != 0)
    {
      for (i=0; i<payload_length; i++)
        dst[o++] = payload[i];
    }
    
    return dst;  
  }
  
  /*
   * writeNdefOnType4
   * ----------------
   * Write a NDEF Message (or a single Record) onto a Type 4 Tag
   */  
  public static boolean writeNdefOnType4(CardChannel channel, byte[] ndef) throws CardException
  {
    /* Select the NDEF application */
    /* --------------------------- */
    
    if (!SelectApplication(channel, NDEF_APPLICATION_ID))
    {
      System.out.println("Failed to Select the NDEF application");
      return false;
    }
    
    /* Read the CC file */
    /* ---------------- */
    
    if (!SelectFile(channel, NDEF_CC_FILE_ID))
    {
      System.out.println("Failed to Select the CC file " + toHex(NDEF_CC_FILE_ID));
      return false;
    }
    byte[] cc_content = ReadBinary(channel, 0, 15);
    if (cc_content ==  null)
    {
      System.out.println("Failed to Read the CC file");
      return false;
    }
    
    /* Parse the CC file */
    /* ----------------- */
    
    int l = cc_content[0] * 0x0100 + cc_content[1];
    if (l < 15)
    {
      System.out.println("Invalid CC file");
      return false;    
    }
    
    if ((cc_content[2] & 0xF0) != 0x20)
    {
      System.out.println("Bad version in the CC file");
      return false;        
    }
  
	  int max_le = (cc_content[3] * 0x0100 + cc_content[4]);
		int max_lc = (cc_content[5] * 0x0100 + cc_content[6]);
  
    if (cc_content[7] != NDEF_FILE_CONTROL_TLV)
    {
      System.out.println("Bad TLV's Tag in the CC file");
      return false;
    }
    
    if (cc_content[8] < 6)
    {
      System.out.println("Bad TLV's Length in the CC file");
      return false;
    }
    
    byte ndef_file_id[] = new byte[2];
    ndef_file_id[0] = cc_content[9];
    ndef_file_id[1] = cc_content[10];
    int ndef_file_size = (cc_content[11] * 0x0100 + cc_content[12]);
    
    /* Select the NDEF file */
    /* -------------------- */
  
    if (!SelectFile(channel, ndef_file_id))
    {
      System.out.println("Failed to Select the NDEF file " + toHex(ndef_file_id));
      return false;
    }
    
    /* Write the NDEF content */
    /* ---------------------- */
    
    int offset_in_file = 2;
    int offset_in_content = 0;
    
    while (offset_in_content < ndef.length)
    {
      byte[] buffer;
      
      if ((ndef.length - offset_in_content) > max_lc)
      {
        buffer = new byte[max_lc];
      }
      else if ((ndef.length - offset_in_content) > 254)
      {
        buffer = new byte[254];
      } else
      {
        buffer = new byte[ndef.length - offset_in_content];
      }
      
      for (int i=0; i<buffer.length; i++)
        buffer[i] = ndef[offset_in_content++];
      
      if (!WriteBinary(channel, offset_in_file, buffer))
      {
        System.out.println("Failed to write the NDEF file at offset " + offset_in_file);
        return false;
      }
      
      offset_in_file += buffer.length;
    }
    
    /* Write NDEF length as header */
    /* --------------------------- */

    byte[] buffer = new byte[2];
    
    buffer[0] = (byte) (ndef.length  / 0x0100);
    buffer[1] = (byte) (ndef.length  % 0x0100);

    if (!WriteBinary(channel, 0, buffer))
    {
      System.out.println("Failed to write the header in the NDEF file");
      return false;
    }
    
    return true;
  }  

  /*
   * ReadBinary
   * ----------
   */
  public static byte[] ReadBinary(CardChannel channel, int offset, int length) throws CardException
  {
    CommandAPDU command = new CommandAPDU(
      0x00, /* CLA */
      0xB0, /* INS */
      (offset / 0x0100), /* P1 */
      (offset % 0x0100), /* P2 */
      length /* Le */
    );
    
    System.out.println("# < " + toHex(command.getBytes()));
  
    ResponseAPDU response = channel.transmit(command);
    
    if (response == null)
      return null;
      
    System.out.println("# > " + toHex(response.getBytes()));
    
    if (response == null)
      return null;

    if (response.getSW() != 0x9000)
      return null;
      
    return response.getData();
  }
  
  /*
   * WriteBinary
   * -----------
   */
  public static boolean WriteBinary(CardChannel channel, int offset, byte[] data) throws CardException
  {
    CommandAPDU command = new CommandAPDU(
      0x00, /* CLA */
      0xD6, /* INS */
      (offset / 0x0100), /* P1 */
      (offset % 0x0100), /* P2 */
      data /* Lc and Data */
    );
  
    System.out.println("# < " + toHex(command.getBytes()));
  
    ResponseAPDU response = channel.transmit(command);
    
    if (response == null)
      return false;
      
    System.out.println("# > " + toHex(response.getBytes()));
    
    if (response == null)
      return false;

    if (response.getSW() != 0x9000)
      return false;
      
    return true;
  }
  
  /*
   * SelectApplication
   * -----------------
   */
  public static boolean SelectApplication(CardChannel channel, byte[] aid) throws CardException
  {
    CommandAPDU command = new CommandAPDU(
      0x00, /* CLA */
      0xA4, /* INS */
      0x04, /* P1 */
      0x00, /* P2 */
      aid /* Lc and Data */
    );
  
    System.out.println("# < " + toHex(command.getBytes()));
  
    ResponseAPDU response = channel.transmit(command);
    
    if (response == null)
      return false;
      
    System.out.println("# > " + toHex(response.getBytes()));
    
    if (response == null)
      return false;

    if (response.getSW() != 0x9000)
      return false;
      
    return true;
  }

  /*
   * SelectFile
   * ----------
   */
  public static boolean SelectFile(CardChannel channel, byte[] fid) throws CardException
  {
    CommandAPDU command = new CommandAPDU(
      0x00, /* CLA */
      0xA4, /* INS */
      0x00, /* P1 */
      0x0C, /* P2 */
      fid /* Lc and Data */
    );
    
    System.out.println("# < " + toHex(command.getBytes()));
  
    ResponseAPDU response = channel.transmit(command);
    
    if (response == null)
      return false;
      
    System.out.println("# > " + toHex(response.getBytes()));

    if (response.getSW() != 0x9000)
      return false;
      
    return true;
  }
  
  private static int SCARD_CTL_CODE(int code) {
    int ioctl;
    String os_name = System.getProperty("os.name").toLowerCase();
    if (os_name.indexOf("windows") > -1)
    {
      ioctl = (0x31 << 16 | (code) << 2);
    }
    else
    {
      ioctl = 0x42000000 + (code);
    }
    return ioctl;
  }

  private static int CONTROL_CODE() {
    String os_name = System.getProperty("os.name").toLowerCase();
    if (os_name.indexOf("windows") > -1)
    {
      /* Value used by both MS' CCID driver and SpringCard's CCID driver */
      return SCARD_CTL_CODE(3500);
    }
    else
    {
      /* Value used by PCSC-Lite */
      return SCARD_CTL_CODE(1);
    }
  }
  
  /*
   * SCardControl
   * ------------
   */
  public static boolean sendControlCommand(CardTerminal terminal, byte[] command)
  {
    /* Magic value for SCardControl dwControlCode parameter */
//    int controlCode = 0x310000 + 3500 * 4;
  
    /* Get a DIRECT card handle */
    Card card;
    
    try
    {
      card = terminal.connect("DIRECT");
    }
    catch (CardException e)
    {
      System.out.println("Direct connection to terminal: exception");
      System.out.println("\t" + e.getMessage());
      return false;
    }

    /* Send the Control Command to the terminal */
    byte[] response;
    try
    {    
      response = card.transmitControlCommand(CONTROL_CODE(), command);
    }
    catch (CardException e)
    {
      System.out.println("Send direct command to terminal: exception");
      System.out.println("\t" + e.getMessage());

      try
      {
        card.disconnect(false);
      }
      catch (CardException ep)
      {
      
      }
      return false;
    }
    
    /* Close the card channel */
    try
    {
      card.disconnect(false);
    }
    catch (CardException e)
    {
    
    }
    
    /* Check response (must be 1 byte equal to 0x00) */
    if ((response == null) || (response.length < 1))
    {
      System.out.println("Send direct command to terminal: empty response");
      return false;    
    }    
    if (response[0] == 0)
      return true;
    System.out.println("Send direct command to terminal: invalid response: " + (0 - response[0]));
    return false;    
  }
  
  public static boolean startNfcTagType4Emulation(CardTerminal terminal)
  {
    byte[] START_TAG_TYPE4_EMUL_COMMAND = new byte[] { (byte) 0x83, (byte) 0x10, (byte) 0x04, (byte) 0x11 };
    
    return sendControlCommand(terminal, START_TAG_TYPE4_EMUL_COMMAND);
  }

  public static boolean backToReaderMode(CardTerminal terminal)
  {
    byte[] READER_MODE_COMMAND = new byte[] { (byte) 0x83, (byte) 0x10, (byte) 0x00, (byte) 0x00 };
    
    return sendControlCommand(terminal, READER_MODE_COMMAND);
  }
  
  static String toHex(byte b)
  {
    return String.format("%02X", b);
  }
  
  static String toHex(byte[] bArray)
  {
    String r = "";
    for (byte b:bArray)
      r += toHex(b);
    return r;
  }
}
