﻿/* Created by SharpDevelop.
 * User: johann
 * Date: 02/03/2012
 * Time: 17:56
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using SpringCardPCSC;
using SpringCardNFC;

namespace SpringCardApplication
{
	public partial class MainForm : Form
	{
		private const int MODE_RECV_ONLY = 0;
		private const int MODE_SEND_ONLY = 1;
		private const int MODE_RECV_THEN_SEND = 2;
		private const int MODE_SEND_THEN_RECV = 3;
		
		private const int STEP_FIRST_CONFIG = 0;
		private const int STEP_FIRST_READY = 1;
		private const int STEP_SECOND_CONFIG = 2;
		private const int STEP_SECOND_READY = 3;
		
		ShadowForm shade = null;
		NdefDisplayForm display = null;
		SCardReader reader = null;
		RtdControl control = null;
		
		LLCP llcp = null;
		
		int mode = MODE_RECV_ONLY;
		int step = STEP_FIRST_CONFIG;
		
		bool ready = false;
		
		public MainForm()
		{
			InitializeComponent();
			
			Settings s = new Settings();
			if (s.ShowConsole)
			{
				SpringCardUtils.SystemConsole.Show();
				Trace.Listeners.Add(new ConsoleTraceListener());
			}
			miShowConsole.Checked = s.ShowConsole;
			
			ready = true;
		}

		delegate void ReaderStatusChangedInvoker(uint ReaderState, CardBuffer CardAtr);
		void ReaderStatusChanged(uint ReaderState, CardBuffer CardAtr)
		{
			/* The ReaderStatusChanged function is called as a delegate (callback) by the SCardReader object    */
			/* within its backgroung thread. Therefore we must use the BeginInvoke syntax to switch back from   */
			/* the context of the background thread to the context of the application's main thread. Overwise   */
			/* we'll get a security violation when trying to access the window's visual components (that belong */
			/* to the application's main thread and can't be safely manipulated by background threads).         */
			if (InvokeRequired)
			{
				this.BeginInvoke(new ReaderStatusChangedInvoker(ReaderStatusChanged), ReaderState, CardAtr);
				return;
			}
			
			eReaderStatus.Text = SCARD.ReaderStatusToString(ReaderState);
			
			if (CardAtr != null)
			{
				eCardAtr.Text = CardAtr.AsString(" ");
			} else
			{
				eCardAtr.Text = "";
			}
			
			if (ReaderState == SCARD.STATE_UNAWARE)
			{
				if (llcp != null)
				{
					llcp.Stop();
					llcp = null;
				}
				
				MessageBox.Show("The reader we were working on has disappeared from the system. This application will terminate now.",
				                "The reader has been removed",
				                MessageBoxButtons.OK,
				                MessageBoxIcon.Exclamation);
				
				Application.Exit();
				
			} else
				if ((ReaderState & SCARD.STATE_EMPTY) != 0)
			{
				if (llcp != null)
				{
					llcp.Stop();
					llcp = null;
				}
				
				Trace.WriteLine("*** Card removed ***");
				pControl.Enabled = true;
				
				if (mode == MODE_RECV_ONLY)
				{
					/* We are a receiver */
					if (step == STEP_FIRST_CONFIG)
					{
						AnnounceReceiver(true);
						step = STEP_FIRST_READY;
					}
				}
				else if (mode == MODE_RECV_THEN_SEND)
				{
					if (step == STEP_SECOND_CONFIG)
					{
						/* We've received, we don't want to receive anymore */
						AnnounceReceiver(false);
						step = STEP_SECOND_READY;
					}
				}
				else if (mode == MODE_SEND_ONLY)
				{
					step = STEP_FIRST_READY;
				}
				else if (mode == MODE_SEND_THEN_RECV)
				{
					if (step == STEP_SECOND_CONFIG)
					{
						/* We've sent, we now want to receive */
						AnnounceReceiver(true);
						step = STEP_SECOND_READY;
					}
				}
				

//			  card_status = 0;
			} else
				if ((ReaderState & SCARD.STATE_UNAVAILABLE) != 0)
			{
//			  card_status = 0;
			} else
				if ((ReaderState & SCARD.STATE_MUTE) != 0)
			{
//			  card_status = 0;
			} else
				if ((ReaderState & SCARD.STATE_INUSE) != 0)
			{
//			  card_status = 0;
			} else
				if ((ReaderState & SCARD.STATE_PRESENT) != 0)
			{
				/* Try to work with this card, it may be a LLCP peer */
				pControl.Enabled = false;
				Trace.WriteLine("*** Card arrived ***");
				
				if (llcp == null)
				{
					llcp = new LlcpInitiator(reader);
					
					if (mode == MODE_RECV_ONLY)
					{
						/* We are a receiver --> we start our SNEP server */
						StartReceiver();
						step = STEP_FIRST_READY;
					}
					else if (mode == MODE_RECV_THEN_SEND)
					{
						/* We are a receiver only half time */
						if (step == STEP_FIRST_READY)
						{
							/* This is our receiver half! */
							StartReceiver();
						} else
						{
							/* This is our sender half */
							StartSender();
						}
					}
					else if (mode == MODE_SEND_ONLY)
					{
						/* We are a sender --> we start our SNEP client */
						StartSender();
						step = STEP_FIRST_READY;
					}
					else if (mode == MODE_SEND_THEN_RECV)
					{
						/* We are a sender only half time */
						if (step == STEP_SECOND_READY)
						{
							/* This is our receiver half! */
							StartReceiver();
						} else
						{
							/* This is our sender half! */
							StartSender();
						}
					}
					
					llcp.Start();
				}
				
				
//			  card_status = 1;
			}
		}
		
		void StartReceiver()
		{
			/* Instanciate a SNEP server - if a message arrives, it will call our OnNdefMessageReceived method */
			Trace.WriteLine("Create (and start) SNEP Server (receiver)");
			llcp.RegisterServer(new SNEP_Server(OnNdefMessageReceived));
		}
		
		void StartSender()
		{
			/* Instanciate a SNEP client */
			Ndef myNdef = CreateMyNdefMessage();

			if (myNdef != null)
			{
				Trace.WriteLine("Create SNEP Client (sender)");
				SNEP_Client client = new SNEP_Client(myNdef);
				client.OnMessageSent = new SNEP_Client.SNEPMessageSentCallback(OnNdefMessageSent);
				client.MessageSentCallbackOnAcknowledge = true;
				llcp.RegisterClient(client);
			} else
			{
				Trace.WriteLine("Can't create SNEP Client: nothing to send");
			}
		}

		void MainFormShown(object sender, EventArgs e)
		{
			if (true)
			{
				SplashForm splash = new SplashForm();
				splash.ShowDialog();
			}
			
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			
			if (readerSelect.SelectedReader == null)
			{
				readerSelect.Preselect("SpringCard|NFC|tactless");
				for (;;)
				{
					readerSelect.ShowDialog();
					if (readerSelect.SelectedReader != null)
						break;
					
					if (MessageBox.Show("This application can't run without a reader. Do you want to leave the application now ?",
					                    "No reader selected",
					                    MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
					{
						Application.Exit();
						return;
					}
				}
			}
			
			eReaderName.Text = readerSelect.SelectedReader;
			eReaderStatus.Text = "";
			eCardAtr.Text = "";
			
			shade.Dispose();
			shade = null;
			
			reader = new SCardReader(eReaderName.Text);
			
			Settings s = new Settings();
			
			switch (s.SelectedMode)
			{
				case Settings.MODE_RECV_ONLY :
					rbRecvOnly.Checked = true;
					break;
				case Settings.MODE_SEND_ONLY :
					rbSendOnly.Checked = true;
					break;
				case Settings.MODE_RECV_THEN_SEND :
					rbRecvThenSend.Checked = true;
					break;
				case Settings.MODE_SEND_THEN_RECV :
					rbSendThenRecv.Checked = true;
					break;
				case Settings.MODE_SEND_AND_RECV :
					default :
						rbSendThenRecv.Checked = true;
					break;
			}
			
			CbModeCheckedChanged(null, null);
			
			if ((s.SelectedType >= 0) && (s.SelectedType < cbNdefType.Items.Count))
				cbNdefType.SelectedIndex = s.SelectedType;
			else
				cbNdefType.SelectedIndex = 0;
			CbNdefTypeSelectedIndexChanged(null, null);
			
			reader.StartMonitor(new SCardReader.StatusChangeCallback(ReaderStatusChanged));
		}
		
		void MainFormFormClosed(object sender, FormClosedEventArgs e)
		{
			if (llcp != null)
			{
				llcp.Stop();
				llcp = null;
			}

			if (reader != null)
			{
				reader.StopMonitor();
				reader = null;
			}
			
			Settings s = new Settings();
			
			if (rbRecvOnly.Checked) s.SelectedMode = Settings.MODE_RECV_ONLY; else
			if (rbSendOnly.Checked) s.SelectedMode = Settings.MODE_SEND_ONLY; else
			if (rbRecvThenSend.Checked) s.SelectedMode = Settings.MODE_RECV_THEN_SEND; else
				if (rbSendThenRecv.Checked) s.SelectedMode = Settings.MODE_SEND_THEN_RECV; else s.SelectedMode = Settings.MODE_SEND_AND_RECV;

			s.SelectedType = cbNdefType.SelectedIndex;

			s.Save();
		}
		
		void MainFormLoad(object sender, EventArgs e)
		{
			shade = new ShadowForm();
			shade.ShowShadow(this);
		}
		
		void AboutToolStripMenuItemClick(object sender, EventArgs e)
		{
			shade = new ShadowForm();
			shade.ShowShadow(this);
			AboutForm f = new AboutForm();
			f.ShowDialog();
			shade.Dispose();
			shade = null;
		}
		
		
		void ChangeReader()
		{
			if (reader != null)
			{
				reader.StopMonitor();
				reader = null;
			}
			
			eReaderStatus.Text = "";
			eCardAtr.Text = "";

			shade = new ShadowForm();
			shade.ShowShadow(this);
			ReaderSelectForm readerSelect = new ReaderSelectForm();
			readerSelect.SelectedReader = eReaderName.Text;
			readerSelect.ShowDialog();
			if (readerSelect.SelectedReader != null)
				eReaderName.Text = readerSelect.SelectedReader;
			shade.Dispose();
			shade = null;
			
			reader = new SCardReader(eReaderName.Text);

			reader.StartMonitor(new SCardReader.StatusChangeCallback(ReaderStatusChanged));
		}
		
		void MainFormFormClosing(object sender, FormClosingEventArgs e)
		{

		}
		
		void imgLogoClick(object sender, EventArgs e)
		{
			System.Diagnostics.Process.Start("http://www.springcard.com");
		}
		
		void BtnReaderChangeClick(object sender, System.EventArgs e)
		{
			ChangeReader();
		}
		
		void QuitToolStripMenuItemClick(object sender, EventArgs e)
		{
			Close();
		}
		
		Ndef CreateMyNdefMessage()
		{
			if (control == null)
				return null;
			
			return control.GetContent();
		}
		

		delegate void OnNdefMessageReceivedInvoker(Ndef hisNdef);
		void OnNdefMessageReceived(Ndef hisNdef)
		{
			/* The OnNdefMessageReceived function is called as a delegate (callback) within the NfcLlcp         */
			/* object's backgroung thread. Therefore we must use the BeginInvoke syntax to switch back from     */
			/* the context of the background thread to the context of the application's main thread. Overwise   */
			/* we'll get a security violation when trying to access the window's visual components (that belong */
			/* to the application's main thread and can't be safely manipulated by background threads).         */
			if (InvokeRequired)
			{
				this.BeginInvoke(new OnNdefMessageReceivedInvoker(OnNdefMessageReceived), hisNdef);
				return;
			}
			
			/* Message received, what shall we do now ? */
			/* ---------------------------------------- */
			
			Trace.WriteLine("*** NDEF message received ***");
			
			if (mode == MODE_RECV_THEN_SEND)
			{
				/* Going to send */
				step = STEP_SECOND_CONFIG;
				/* Reset, so the smartphone will start a new LLCP session */
				llcp.StopAndResetPeer();
			}
			else if (mode == MODE_SEND_THEN_RECV)
			{
				/* Back to send mode */
				step = STEP_FIRST_CONFIG;
			}
			else if (mode == MODE_RECV_ONLY)
			{

			}
			
			
			if (display != null)
			{
				/* We already are displaying a message, we must wait until the user closes the form */
				return;
			}
			
			display = new NdefDisplayForm();
			
			//  	  string t = (new CardBuffer(hisNdef.GetBytes())).AsString(" ");
			//  	  MessageBox.Show(t);

			shade = new ShadowForm();
			shade.ShowShadow(this);

			display.SetNdef(hisNdef);
			display.ShowDialog();
			
			shade.Dispose();
			shade = null;

			display = null;
		}
		
		delegate void OnNdefMessageSentInvoker();
		void OnNdefMessageSent()
		{
			if (InvokeRequired)
			{
				this.BeginInvoke(new OnNdefMessageSentInvoker(OnNdefMessageSent));
				return;
			}
			
			Trace.WriteLine("*** NDEF message sent ***");

			if (mode == MODE_SEND_THEN_RECV)
			{
				/* Going to receive */
				step = STEP_SECOND_CONFIG;
				/* Reset, so the smartphone will start a new LLCP session */
				llcp.StopAndResetPeer();
			} else
				if (mode == MODE_RECV_THEN_SEND)
			{
				/* Back to receive mode */
				step = STEP_FIRST_CONFIG;
			}
		}
		
		void CbNdefTypeSelectedIndexChanged(object sender, EventArgs e)
		{
			if (control != null)
			{
				pMain.Controls.Remove(control);
				control.Dispose();
				control = null;
			}
			
			switch (cbNdefType.SelectedIndex)
			{
				case 0:
					/* Text */
					control = new RtdTextControl();
					break;

				case 1:
					/* URI */
					control = new RtdUriControl();
					break;

				case 2:
					/* SmartPoster */
					control = new RtdSmartPosterControl();
					break;

				case 3:
					/* vCard */
					control = new RtdVCardControl();
					break;

				case 4:
					/* Arbitrary MIME Media (text) */
					control = new RtdMediaControl();
					break;
					
				case 5:
					/* WifiHandover */
					control = new RtdWifiHandoverControl();
					break;
										
				default :
					break;
			}
			
			if (control != null)
			{
				control.Dock = DockStyle.Fill;
				pMain.Controls.Add(control);
			}
		}
		
		void MiShowConsoleCheckedChanged(object sender, EventArgs e)
		{
			if (!ready)
				return;
			
			Settings s = new Settings();
			s.ShowConsole = miShowConsole.Checked;
			s.Save();
			if (miShowConsole.Checked)
			{
				SpringCardUtils.SystemConsole.Show();
				Trace.Listeners.Add(new ConsoleTraceListener());
			} else
			{
				Trace.Listeners.Clear();
				SpringCardUtils.SystemConsole.Hide();
			}
		}
		
		bool AnnounceReceiver(bool yes)
		{
			if (!LlcpInitiator.AnnounceSnepServer(reader, yes))
			{
				MessageBox.Show("Warning: problem with ATR GI");
				return false;
			}
			return true;
		}
		
		void CbModeCheckedChanged(object sender, EventArgs e)
		{
			if (rbSendOnly.Checked)
			{
				mode = MODE_SEND_ONLY;
				AnnounceReceiver(false);
			}
			else if (rbSendThenRecv.Checked)
			{
				mode = MODE_SEND_THEN_RECV;
				AnnounceReceiver(false);
			}
			else if (rbRecvOnly.Checked)
			{
				mode = MODE_RECV_ONLY;
				AnnounceReceiver(true);
			}
			else if (rbRecvThenSend.Checked)
			{
				mode = MODE_RECV_THEN_SEND;
				AnnounceReceiver(true);
			}
			
			step = STEP_FIRST_READY;
			
			if ((mode == MODE_SEND_ONLY) || (mode == MODE_SEND_THEN_RECV) || (mode == MODE_RECV_THEN_SEND))
			{
				/* We could send */
				cbNdefType.Enabled = true;
				pMain.Enabled = true;
			} else
			{
				/* We couldn't */
				cbNdefType.Enabled = false;
				pMain.Enabled = false;
			}
		}
	}
}
