/**h* NFCTools/nfc_rtd_uri.c
 *
 * NAME
 *   nfc_rtc_uri.c
 *
 * DESCRIPTION
 *   Implementation of the NFC RTD URI
 *
 * COPYRIGHT
 *   (c) 2011 PRO ACTIVE SAS - See LICENCE.txt for licence information
 *
 * AUTHOR
 *   Johann Dantant / PRO ACTIVE for www.springcard.com
 *
 * HISTORY
 *   JDA 17/01/2011 : first public release
 *
 **/
#include "nfc.h"

typedef struct
{
  unsigned char header;
  const char *str;
} _uri_prefix_t;

static const _uri_prefix_t prefixes[] =
{
  { 0x01, "http://www." },
  { 0x02, "https://www." },
  { 0x03, "http://" },
  { 0x04, "https://" },
  { 0x05, "tel:" },
  { 0x06, "mailto:" },
  { 0x07, "ftp://anonymous:anonymous@" },
  { 0x08, "ftp://ftp." },
  { 0x09, "ftps://" },
  { 0x0A, "sftp://" },
  { 0x0B, "smb://" },
  { 0x0C, "nfs://" },
  { 0x0D, "ftp://" },
  { 0x0E, "dav://" },
  { 0x0F, "news:" },
  { 0x10, "telnet://" },
  { 0x11, "imap:" },
  { 0x12, "rtsp://" },
  { 0x13, "urn:" },
  { 0x14, "pop:" },
  { 0x15, "sip:" },
  { 0x16, "sips:" },
  { 0x17, "tftp:" },
  { 0x18, "btspp://" },
  { 0x19, "btl2cap://" },
  { 0x1A, "btgoep://" },
  { 0x1B, "tcpobex://" },
  { 0x1C, "irdaobex://" },
  { 0x1D, "file://" },
  { 0x1E, "urn:epc:id" },
  { 0x1F, "urn:epc:tag" },
  { 0x20, "urn:epc:pat" },
  { 0x21, "urn:epc:raw" },
  { 0x22, "urn:epc:" },
  { 0x23, "urn:nfc:" },
};

BOOL uri_encode_alloc_a(unsigned char **dst, size_t *dst_sz, const char *src)
{
  unsigned int i;
  BYTE header = 0;
	size_t pfx_sz = 0;
  size_t src_sz = 0;

	if (src != NULL)
    src_sz = strlen(src);

  for (i=0; i<(sizeof(prefixes)/sizeof(prefixes[0])); i++)
  {
	  pfx_sz = strlen(prefixes[i].str);
    if ((src_sz > pfx_sz) && !strncmp(src, prefixes[i].str, pfx_sz))
		{
		  header = prefixes[i].header;
			break;
		}
	}
	if (header == 0)
	  pfx_sz = 0;

	if (dst_sz != NULL)
	  *dst_sz = 1 + src_sz - pfx_sz;

	if (dst != NULL)
	{
    unsigned char *_dst = nfc_alloc(1 + src_sz - pfx_sz);

		_dst[0] = header;

		if (src_sz)
		  memcpy(&_dst[1], &src[pfx_sz], src_sz - pfx_sz);

		*dst = _dst;
	}

  return TRUE;
}

BOOL uri_decode_alloc_a(char **dst, unsigned char *src, size_t src_sz)
{
  unsigned int i;

  if ((dst == NULL) || (src == NULL) || (src_sz < 1))
    return FALSE;

  if (src[0] == 0x00)
  {
    *dst = nfc_alloc(src_sz);
    memcpy(&(*dst)[0], &src[1], src_sz - 1);
    return TRUE;
  }
  
  for (i=0; i<(sizeof(prefixes)/sizeof(prefixes[0])); i++)
  {
    if (src[0] == prefixes[i].header)
    {
      size_t pfx_len = strlen(prefixes[i].str);

      *dst = nfc_alloc(pfx_len + src_sz);     
      memcpy(&(*dst)[0], prefixes[i].str, pfx_len);
      memcpy(&(*dst)[pfx_len], &src[1], src_sz - 1);
      return TRUE;
    }
  }

  *dst = nfc_alloc(3 + src_sz);
  sprintf(&(*dst)[0], "%02X|", src[0]);
  memcpy(&(*dst)[3], &src[1], src_sz - 1);
  return TRUE;
}
