/**h* NFCTools/nfc_tag.c
 *
 * NAME
 *   nfc_tag.c
 *
 * DESCRIPTION
 *   Abstract NFC Tag object
 *
 * COPYRIGHT
 *   (c) 2011 PRO ACTIVE SAS - See LICENCE.txt for licence information
 *
 * AUTHOR
 *   Johann Dantant / PRO ACTIVE for www.springcard.com
 *
 * HISTORY
 *   JDA 17/01/2011 : first public release
 *
 **/
#include "nfc.h"
#include "nfc_tag_i.h"

nfc_tag_t nfc_tac_alloc()
{
  struct _nfc_tag_st *tag = nfc_alloc(sizeof(struct _nfc_tag_st));
  return tag;
}

void nfc_tag_clear(nfc_tag_t tag)
{
  if (tag != NULL)
  {
    if (tag->type == NFC_TAG_SMARTPOSTER)
		  nfc_smartposter_clear(tag);

    memset(tag, 0, sizeof(*tag));
  }
}

void nfc_tag_free(nfc_tag_t tag)
{
  if (tag != NULL)
  {
    nfc_tag_clear(tag);
    nfc_free(tag);
  }
}

int nfc_tag_get_type(nfc_tag_t tag)
{
  if (tag == NULL)
    return 0;
  return tag->type;
}

BOOL nfc_tag_load_from_file(nfc_tag_t tag, char *file_name)
{
  size_t size;
  unsigned char *buffer;

  if ((tag == NULL) || (file_name == NULL))
    return FALSE;

  nfc_tag_clear(tag);

  if (nfc_verbose)
    printf("nfc_tag.load_from_file(%s)\n", file_name);

  if (!nfc_file_exists(file_name))
  {
    if (!nfc_silent)
      printf("Input file not found\n");
    return FALSE;
  }

  size = nfc_file_size(file_name);
  if (size == 0)
  {
    if (!nfc_silent)
      printf("Input file is empty\n");
    return FALSE;
  }
  if (size > MAX_NFC_TAG_SIZE)
  {
    if (!nfc_silent)
      printf("Input file is too big\n");
    return FALSE;
  }

  buffer = nfc_alloc(size);
  if (buffer == NULL)
    return FALSE;

  if (!nfc_read_file(buffer, size, file_name))
  {
    if (!nfc_silent)
      printf("Read from input file failed\n");
    nfc_free(buffer);
    return FALSE;
  }

  if (!ndef_parse_tag(tag, buffer, size))
  {
	  printf("Read Tag from file: content is invalid\n");
    nfc_free(buffer);
    return FALSE;
  }

  nfc_free(buffer);
  return TRUE;
}

BOOL nfc_tag_save_to_file(nfc_tag_t tag, char *file_name)
{
  unsigned char *buffer;
  size_t size;

  if ((tag == NULL) || (file_name == NULL))
    return FALSE;

  if (!ndef_assemble_tag_alloc(tag, &buffer, &size))
  {
    if (!nfc_silent)
      printf("Write Tag to file: can't assemble the output\n");
    return FALSE;
  }

  if (!nfc_write_file(buffer, size, file_name))
	{
    if (!nfc_silent)
      printf("Write Tag to file: failed\n");
		nfc_free(buffer);
    return FALSE;
  }

	nfc_free(buffer);
  return TRUE;
}

