/**h* NFCTools/nfc_tag_smartposter.c
 *
 * NAME
 *   nfc_tag_smartposter.c
 *
 * DESCRIPTION
 *   Implementation of the NFC SmartPoster
 *
 * COPYRIGHT
 *   (c) 2011 PRO ACTIVE SAS - See LICENCE.txt for licence information
 *
 * AUTHOR
 *   Johann Dantant / PRO ACTIVE for www.springcard.com
 *
 * HISTORY
 *   JDA 17/01/2011 : first public release
 *
 **/
#include "nfc.h"
#include "nfc_tag_i.h"

void nfc_smartposter_clear(nfc_tag_t tag)
{
  if ((tag != NULL) && (tag->type == NFC_TAG_SMARTPOSTER))
  {
		nfc_smartposter_clear_titles(tag);
		nfc_smartposter_clear_icon(tag);

    nfc_free(tag->content.smartposter.uri);
    nfc_free(tag->content.smartposter.mime_type);
  }
}

BOOL nfc_smartposter_init(nfc_tag_t tag)
{
  if (tag == NULL)
    return FALSE;

  nfc_tag_clear(tag);
  tag->type = NFC_TAG_SMARTPOSTER;
  tag->content.smartposter.action = -1;
  return TRUE;
}

BOOL nfc_smartposter_set_uri(nfc_tag_t tag, char *uri)
{
  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  if (nfc_verbose)
    printf("smartposter.set_uri(%s)\n", (uri != NULL) ? uri : "");

  nfc_free(tag->content.smartposter.uri);
  tag->content.smartposter.uri = nfc_strdup(uri);

  return TRUE;
}

BOOL nfc_smartposter_clear_titles(nfc_tag_t tag)
{
  _nfc_title_t *title;

  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

	title = tag->content.smartposter.titles;
	tag->content.smartposter.titles = NULL;

	while (title != NULL)
	{
    _nfc_title_t *next_title = title->next;
		nfc_free(title);
		title = next_title;
  }

	return TRUE;
}

BOOL nfc_smartposter_set_title(nfc_tag_t tag, char *text, char *lang)
{
  _nfc_title_t **title_place, *title;

  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  if (nfc_verbose)
    printf("smartposter.set_title(%s, %s)\n", (text != NULL) ? text : "", (lang != NULL) ? lang : "");

  title_place = &tag->content.smartposter.titles;
	title = tag->content.smartposter.titles;

	while (title != NULL)
	{
	  if ((lang == NULL) && (title->lang == NULL))
		{
      nfc_free(title->text);
			title->text = nfc_strdup(text);
			return TRUE;
		}

    if ((lang != NULL) && (title->lang != NULL) && (!strcmp(lang, title->lang)))
		{
      nfc_free(title->text);
			title->text = nfc_strdup(text);
			return TRUE;
		}

    title_place = &title->next;
    title = title->next;
	}

	title = nfc_alloc(sizeof(_nfc_title_t));
	*title_place = title;

	title->next = NULL;
	title->lang = nfc_strdup(lang);
	title->text = nfc_strdup(text);

	return TRUE;
}

BOOL nfc_smartposter_set_action(nfc_tag_t tag, int action)
{
  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  if (nfc_verbose)
  {
    printf("smartposter.set_action(%d)\n", action);
  }

  tag->content.smartposter.action = action;

  return TRUE;
}

BOOL nfc_smartposter_set_size(nfc_tag_t tag, size_t size)
{
  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  if (nfc_verbose)
  {
    printf("smartposter.set_size(%ul)\n", size);
  }

  tag->content.smartposter.size = size;

  return TRUE;
}

BOOL nfc_smartposter_set_mime_type(nfc_tag_t tag, char *mime_type)
{
  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  if (nfc_verbose)
    printf("smartposter.set_mime_type(%s)\n", (mime_type != NULL) ? mime_type : "");

  nfc_free(tag->content.smartposter.mime_type);
  tag->content.smartposter.mime_type = nfc_strdup(mime_type);

  return TRUE;
}

BOOL nfc_smartposter_clear_icon(nfc_tag_t tag)
{
  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  tag->content.smartposter.icon_size = 0;
  nfc_free(tag->content.smartposter.icon_blob);
  tag->content.smartposter.icon_blob = NULL;
  nfc_free(tag->content.smartposter.icon_mime_type);
	tag->content.smartposter.icon_mime_type = NULL;
  
  return TRUE;
}

BOOL nfc_smartposter_load_icon(nfc_tag_t tag, char *icon_file_name)
{
  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  nfc_smartposter_clear_icon(tag);

	if (!nfc_file_exists(icon_file_name))
	{
    if (!nfc_silent)
      printf("Icon file not found\n");
    return FALSE;
	}

  tag->content.smartposter.icon_size = nfc_file_size(icon_file_name);
  if (tag->content.smartposter.icon_size == 0)
	{
    if (!nfc_silent)
      printf("Icon file is empty\n");
    return FALSE;
	}
  if (tag->content.smartposter.icon_size > MAX_NFC_TAG_SIZE)
  {
    if (!nfc_silent)
      printf("Icon file is too big\n");
		tag->content.smartposter.icon_size = 0;
    return FALSE;
  }

  tag->content.smartposter.icon_blob = nfc_alloc(tag->content.smartposter.icon_size);

  if (!nfc_read_file(tag->content.smartposter.icon_blob, tag->content.smartposter.icon_size, icon_file_name))
  {
    if (!nfc_silent)
      printf("Read froml Icon file failed\n");
    nfc_free(tag->content.smartposter.icon_blob);
    tag->content.smartposter.icon_blob = NULL;
    tag->content.smartposter.icon_size = 0;
    return FALSE;
  }

  return TRUE;
}

BOOL nfc_smartposter_set_icon_mime_type(nfc_tag_t tag, char *icon_mime_type)
{
  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  if (nfc_verbose)
    printf("smartposter.set_icon_mime_type(%s)\n", (icon_mime_type != NULL) ? icon_mime_type : "");

  nfc_free(tag->content.smartposter.icon_mime_type);
  tag->content.smartposter.icon_mime_type = nfc_strdup(icon_mime_type);

  return TRUE;
}

BOOL nfc_smartposter_set_icon_blob(nfc_tag_t tag, unsigned char *icon_blob, size_t icon_size)
{
  if ((tag == NULL) || (tag->type != NFC_TAG_SMARTPOSTER))
    return FALSE;

  if ((icon_size != 0) && (icon_blob == NULL))
    return FALSE;

  tag->content.smartposter.icon_size = 0;

  nfc_free(tag->content.smartposter.icon_blob);
  tag->content.smartposter.icon_blob = NULL;

  if (icon_blob != NULL)
  {
    tag->content.smartposter.icon_blob = nfc_alloc(icon_size);
    memcpy(tag->content.smartposter.icon_blob, icon_blob, icon_size);
    tag->content.smartposter.icon_size = icon_size;
  }

  return TRUE;
}
