/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2012 PRO ACTIVE SAS - www.springcard.com
   
  ref_p2p_initiator.c
  -------------------

*/
#include "../_common/pcsc_helpers.h"
#include "p2p_common.h"

BYTE snep_feature = 0;

static SCARDHANDLE ghCard;

BOOL main_ex(SCARDCONTEXT hContext, const char *szReader, const BYTE atr[], DWORD atrlen)
{
  SCARDHANDLE hCard;
  DWORD dwProtocol;
  LONG rc;

  LLCP_FRAME_ST my_frame;
  LLCP_FRAME_ST his_frame;


	UNUSED_PARAMETER(atr);
	UNUSED_PARAMETER(atrlen);

  /* Connect to the target */
  rc = SCardConnect(hContext, szReader, SCARD_SHARE_EXCLUSIVE, SCARD_PROTOCOL_T1, &hCard, &dwProtocol);
  if (rc != SCARD_S_SUCCESS)
	{
    printf("SCardConnect error %08lX\n", rc);
		return FALSE;
	}

  if (snep_feature == CLIENT)
  {
    /* We are a demanding client and try to connect to the SNEP server immediately */
    my_frame.dsap  = LLCP_ADDR_SNEP;
    my_frame.ssap  = LLCP_ADDR_CLIENT;
    my_frame.ptype = LLCP_PTYPE_CONNECT;
    my_frame.payload_length = 0;
    if (!LLCP_SenderEnqueue(&my_frame))
    {
      printf("Failed to enqueue 1st SNEP client request\n");
      SCardDisconnect(hCard, SCARD_RESET_CARD);
      return FALSE;
    }
  }

  /* The functions from p2p_common.c don't convey the handle, so we store a copy locally */
  ghCard = hCard;

  for (;;)
  {
    /* Get the next message we have to send */
    if (!LLCP_SenderDequeue(&my_frame))
    {
      printf("LLCP_SenderDequeue failed\n");
      break;
    }

    /* Perform the exchange: send our frame, receive peer's frame */
    if (!LLCP_Exchange(&my_frame, &his_frame))
    {
      /* Peer has been lost */
      break;
    }

    /* Handle peer's frame */
    if (!LLCP_Handle(&his_frame))
    {
      printf("LLCP_Handle failed\n");
      break;
    }
  }

	/* Disconnect from the target */
  SCardDisconnect(hCard, SCARD_RESET_CARD);
  return TRUE;
}


int main(int argc, char **argv)
{
	SCARDCONTEXT      hContext;
  DWORD  dwReaders, dwReadersOld = 0;

  /* Constant MAXIMUM_SMARTCARD_READERS is only 10 in Windows SDK */
  /* This is not enough for most of our tests. 64 sounds good...  */
  SCARD_READERSTATE *rgscState;

	LPSTR  szReaders = NULL;
  DWORD  dwReadersSz;
  
  DWORD  maxReaders;

	BOOL   bFirstRun = TRUE;
  /*
	BOOL   bShowAtrData = TRUE;
  BOOL   bShowCardData = TRUE;
  BOOL   bShowReaderData = TRUE;
  BOOL   bTestReader = FALSE;
  */

  DWORD  i, j;
	LONG   rc;

  /* Constant MAXIMUM_SMARTCARD_READERS is only 10 in Windows SDK */
  /* This is not enough for most of our tests. 64 sounds good...  */
  maxReaders = MAXIMUM_SMARTCARD_READERS;
  if (maxReaders<64) maxReaders = 64;

  for (i=1; (int) i<argc; i++)
  {
    if (!strcmp(argv[i], "-c")) snep_feature |= CLIENT;
		if (!strcmp(argv[i], "-s")) snep_feature |= SERVER;
  }
  if (!snep_feature)
    snep_feature = SERVER_THEN_CLIENT;

  /*
   * Welcome message, check parameters
   * ---------------------------------
   */
  printf("\n");
  printf("SpringCard PC/SC SDK : NFC peer-to-peer demo (initiator mode)\n");
  printf("-------------------------------------------------------------\n");
  printf("v.210912 (c) 2012 Pro Active\n");
	printf("Press <Ctrl+C> to exit\n\n");

  rgscState = calloc(maxReaders, sizeof(SCARD_READERSTATE));
  if (rgscState == NULL)
  {
    printf("Out of memory\n");
    return EXIT_FAILURE;
  }

	/*
   * Get a handle to the PC/SC resource manager
   * ------------------------------------------
   */
	rc = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &hContext);
	if(rc != SCARD_S_SUCCESS)
	{
		printf("SCardEstablishContext :%lXh\n",rc);
    free(rgscState);
		return EXIT_FAILURE;
	}

	/*
   * Infinite loop, we'll exit only when killed by Ctrl+C
   * ----------------------------------------------------
   */
	for (;;)
	{
  	/*
     * Get the list of available readers
  	 */

    dwReadersSz = SCARD_AUTOALLOCATE;
		rc = SCardListReaders(hContext,
                          NULL,                /* Any group */
                          (LPTSTR) &szReaders, /* Diabolic cast for buffer auto-allocation */
                          &dwReadersSz);       /* Beg for auto-allocation */
	  
		if (rc == SCARD_E_NO_READERS_AVAILABLE)
		{
		  /* No reader at all */
			dwReaders = 0;
		} else
    if (rc != SCARD_S_SUCCESS)
		{
      printf("SCardListReaders error %lXh\n",rc);
			break;
		} else
		{
			/* Track events on found readers. */
			LPTSTR pReader = szReaders;

			for (dwReaders=0; dwReaders<maxReaders; dwReaders++)
			{
        /* End of multi-string array */
				if (*pReader == '\0')
					break;

        /* Remember this reader's name */
				rgscState[dwReaders].szReader = pReader;

        /* Jump to next entry in multi-string array */
				pReader += strlen(pReader) + 1;
			}
		}

		if (bFirstRun)
    {
      /* Program startup, display the number of readers */
      if (dwReaders == 0)
      {
        printf("No PC/SC reader\n\n");
      } else
      {
        printf("%ld PC/SC reader%s found\n\n", dwReaders, dwReaders ? "s" : "");
      }
    }
      
		if (dwReadersOld != dwReaders)
		{
			/* Reader added, or reader removed           */
      /* (re)set the initial state for all readers */
			for (i=0; i<dwReaders; i++)
				rgscState[i].dwCurrentState = SCARD_STATE_UNAWARE;

      if (!bFirstRun)
      {
        int c;

        /* Not the program startup, explain the event */
				if (dwReadersOld > dwReaders)
				{
          c = (int) dwReadersOld-dwReaders;
					printf("%d reader%s ha%s been removed from the system\n\n", c, (c==1)?" ":"s", (c==1)?"s":"ve");
				}
				else
				{
          c = (int) dwReaders-dwReadersOld;
					printf("%d reader%s ha%s been added to the system\n\n", c, (c==1)?" ":"s", (c==1)?"s":"ve");
				}
			}

      dwReadersOld = dwReaders;
		}

    if (dwReaders == 0)
    {
      /* We must wait here, because the SCardGetStatusChange doesn't wait  */
      /* at all in case there's no reader in the system. Silly, isn't it ? */
#ifdef WIN32
      Sleep(1000);
#else
      sleep(1);
#endif
    }

		/*
     * Interesting part of the job : call SCardGetStatusChange to monitor all changes
     * taking place in the PC/SC reader(s)
     */
		rc = SCardGetStatusChange(hContext,
			                        INFINITE,
			                        rgscState,
			                        dwReaders);	
		
		if (rc != SCARD_S_SUCCESS)
    {
      printf("SCardGetStatusChange error %lXh\n",rc);

      if (rc == SCARD_F_INTERNAL_ERROR)
      {
        /* Oups ? Restard from scratch after 2500ms */
        if (szReaders != NULL)
        {
          SCardFreeMemory(hContext, szReaders);
          szReaders = NULL;
        }
#ifdef WIN32
        Sleep(3000);
#else
        sleep(3);
#endif
        continue;
      }
      break;
    }

		for (i=0; i<dwReaders; i++)
		{
      BOOL just_inserted = FALSE;

      if (rgscState[i].dwEventState & SCARD_STATE_CHANGED)
      {
        /* Something changed since last loop        */
        if ( (rgscState[i].dwEventState & SCARD_STATE_PRESENT)
         && !(rgscState[i].dwCurrentState & SCARD_STATE_PRESENT) )
          just_inserted = TRUE;

        /* Remember new current state for next loop */
        rgscState[i].dwCurrentState = rgscState[i].dwEventState;
      } else
      {
        /* Nothing new, don't display anything for this reader */
        /* (unless we're in the first run)                     */
        if (!bFirstRun)
          continue;
      }

      /*
       * Display current reader's state
       * ------------------------------
       */

			/* Reader's name */
      printf("Reader %ld: %s\n", i, rgscState[i].szReader);

      /* Complete status */
      printf("\tCard state:\n");

      if (rgscState[i].dwEventState & SCARD_STATE_IGNORE)
        printf("\t\tIgnore this reader\n");

      if (rgscState[i].dwEventState & SCARD_STATE_UNKNOWN)
        printf("\t\tReader unknown\n");

      if (rgscState[i].dwEventState & SCARD_STATE_UNAVAILABLE)
        printf("\t\tStatus unavailable\n");

      if (rgscState[i].dwEventState & SCARD_STATE_EMPTY)
        printf("\t\tNo card in the reader\n");

      if (rgscState[i].dwEventState & SCARD_STATE_PRESENT)
        printf("\t\tCard present\n");

      if (rgscState[i].dwEventState & SCARD_STATE_ATRMATCH)
        printf("\t\tATR match\n");

      if (rgscState[i].dwEventState & SCARD_STATE_INUSE)
        printf("\t\tCard (or reader) in use\n");

      if (rgscState[i].dwEventState & SCARD_STATE_EXCLUSIVE)
        printf("\t\tCard (or reader) reserved for exclusive use\n");

      if (rgscState[i].dwEventState & SCARD_STATE_MUTE)
        printf("\t\tCard is mute\n");

      if (just_inserted)
      {
        /* Display card's ATR */
        if (rgscState[i].cbAtr)
			  {
				  printf("\tCard ATR:");
				  for (j=0; j<rgscState[i].cbAtr; j++)
					  printf(" %02X", rgscState[i].rgbAtr[j]);
          printf("\n");
        }

        /* Try to communicate with this card, it MAY be a P2P target */
        main_ex(hContext, rgscState[i].szReader, rgscState[i].rgbAtr, rgscState[i].cbAtr);
      }
		}

    /* Free the list of readers  */
    if (szReaders != NULL)
    {
      SCardFreeMemory(hContext, szReaders);
      szReaders = NULL;
    }
      
    /* Not the first run anymore */
		bFirstRun = FALSE;
	}


	rc = SCardReleaseContext(hContext);

  /* Never go here (Ctrl+C kill us before !) */
  free(rgscState);
  return EXIT_SUCCESS;
}


/*
 * LLCP_ExchangeEx
 * ----------------
 * Exchange frames in NFC-DEP
 * This function uses SCardTransmit as we are in initiator mode over PC/SC
 * We must encapsulate the frames in an ENCAPSULATE command (FF FE 00 00 ... ),
 * because the LLCP frames are not ISO 7816-4 compliant, and therefore are
 * rejected by the PC/SC library on some systems.
 */
BOOL LLCP_ExchangeEx(const BYTE send_buffer[], DWORD send_len, BYTE recv_buffer[], DWORD *recv_len)
{
  BYTE   c_apdu[264]; /* Buffer to pass the command to the target */
	BYTE   r_apdu[264]; /* Buffer to receive the response from the target */
	DWORD  c_length = 0;
	DWORD  r_length = sizeof(r_apdu);

  LONG rc;

  if ((send_buffer == NULL) && (send_len != 0))
    return FALSE;

  /* CLA */
  c_apdu[c_length++] = 0xFF;
  /* INS */
  c_apdu[c_length++] = 0xFE;
  /* P1 */
  c_apdu[c_length++] = 0x00;
  /* P2 */
  c_apdu[c_length++] = 0x00;

  if (send_len)
  {
    /* Lc and data */
    c_apdu[c_length++] = (BYTE) send_len;
    memcpy(&c_apdu[c_length], send_buffer, send_len);
    c_length += send_len;
  }

  /* Le : accept any length */
  c_apdu[c_length++] = 0x00;

  /* Actual exchange */  
  rc = SCardTransmit(ghCard, SCARD_PCI_T1, c_apdu, c_length, NULL, r_apdu, &r_length);
  if (rc != SCARD_S_SUCCESS)
  {
    printf("SCardTransmit failed, rc=%d (%08lX)\n", rc, rc);
    return FALSE;
  }

  if (r_length < 2)
  {
    printf("SCardTransmit returned %dB only\n", r_length);
    return FALSE;
  }

  if ((r_apdu[r_length-2] != 0x90) || (r_apdu[r_length-1] != 0x00))
  {
    printf("Wrong SW=%02X%02X\n", r_apdu[r_length-2], r_apdu[r_length-1]);
    return FALSE;
  }
  
  memcpy(recv_buffer, r_apdu, r_length-2);
  *recv_len = r_length-2;

  return TRUE;
}