﻿/*
 * Created by SharpDevelop.
 * User: jerome.i
 * Date: 14/06/2013
 * Time: 16:20
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using SpringCardPCSC;
using MemoryCardTool;

namespace SpringCardMemoryCard
{
	/// <summary>
	/// Description of MemoryCard.
	/// </summary>
	public abstract class MemoryCard
	{
		private const string ATR_MEMORY_CARD 	= "3B8F8001804F0CA000000306";
		private const string ATR_MIFARE_1K 		= "3B8F8001804F0CA000000306030001";
		private const string ATR_MIFARE_4K 		= "3B8F8001804F0CA000000306030002";
		protected const byte MIFARE_1K				= 0x40;
		protected const byte MIFARE_4K				= 0x41;
		
		protected SCardChannel _channel = null;
		protected byte type;
		protected string _cardType;
		
		protected MainForm MF;		
		
		public MemoryCard(MainForm _MF, SCardChannel channel)
		{
			MF = _MF;
			_channel = channel;
		}
		
		protected static void log(MainForm _MF, string s)
		{
			if (_MF != null)
				_MF.add_log_text(s);
		}
		
		protected void log(string s)
		{
			if (MF != null)
				MF.add_log_text(s);			
		}
		
		public static bool Recognize(MainForm _MF, SCardChannel cardchannel, out MemoryCard card, out string msg)
		{	
			string atr = cardchannel.CardAtr.AsString("");
			bool recognized = false;
			card = null;
			msg = null;
			
			if (atr.Length < ATR_MEMORY_CARD.Length)
			{
				msg = "This card is not a memory card";
				log(_MF, "This card is not a memory card");
			} else
			{
				if (!atr.Substring(0, ATR_MEMORY_CARD.Length).Equals(ATR_MEMORY_CARD))
				{
					msg = "This card is not a memory card";		
					log(_MF, "This card is not a memory card");
				} else
				{
					/* This is a memory card. Is this a Mifare Classic ?	*/
					if (atr.Length < ATR_MIFARE_1K.Length)
					{
						/* Not a Mifare Classic	*/
						log(_MF, "This is memory card");
						try 
						{
							card = new MemoryCardWithoutSectors(_MF, cardchannel);
						} 
						catch
						{
							msg = "This card can't be read";	
							log(_MF, "This card can't be read");							
							card = null;
						}
						
					} else
					{
						if (atr.Substring(0, ATR_MIFARE_1K.Length).Equals(ATR_MIFARE_1K))
						{
							log(_MF,"This is a Mifare Classic 1k");
							try
							{
								card = new MemoryCardMifareClassic(_MF, cardchannel, MIFARE_1K);
							} catch
							{
								msg = "This card can't be read";	
								log(_MF,"This card can't be read");
								card = null;								
							}
						} else
						if (atr.Substring(0, ATR_MIFARE_4K.Length).Equals(ATR_MIFARE_4K))
						{
							log(_MF,"This is a Mifare Classic 4k");
							try
							{
								card = new MemoryCardMifareClassic(_MF, cardchannel, MIFARE_4K);
							} catch
							{
								msg = "This card can't be read";		
								log(_MF,"This card can't be read");
								card = null;
							}
						} else
						{
							/* Not a Mifare Classic	*/
							log(_MF,"This is memory card");
							try
							{
								card = new MemoryCardWithoutSectors(_MF, cardchannel);
							}
							catch
							{
								msg = "This card can't be read";		
								log(_MF,"This card can't be read");
								card = null;
							}							
						}
					}
					
					recognized = true;					
				}
			}
			
			return recognized;
		}
		

	
		public abstract bool ReadAgain();
		
		
		public string CardType
		{
			get
			{
				return _cardType;
			}
		}
		
	}
	
}
