﻿/*
 * Created by SharpDevelop.
 * User: jerome.i
 * Date: 17/06/2013
 * Time: 09:43
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading;
using SpringCardPCSC;
using MemoryCardTool;

namespace SpringCardMemoryCard
{
	/// <summary>
	/// Description of MemoryCard_Mifare1k.
	/// </summary>
	public class MemoryCardMifareClassic : MemoryCard
	{
		
		private List<MifareSector> sectors;
		private int _nbSectors;
		
		
		public MemoryCardMifareClassic(MainForm _MF, SCardChannel Channel, byte _type) : base(_MF, Channel)
		{
			type = _type;
			
			if (type == MIFARE_1K)
			{	
				_nbSectors = 16;
				_cardType = "Mifare Classic 1k";
				log("Mifare Classic 1k");
			} else 
			if (type == MIFARE_4K)
			{	
				_nbSectors = 40;
				_cardType = "Mifare Classic 4k";
				log("Mifare Classic 4k");
			} else
			{
				/* ? --> 16 by default	*/
				_nbSectors = 16;
				_cardType = "Mifare Classic 1k";
				log("Mifare Classic 1k");
			}
			
			if (!Read())
			{
				throw new System.ApplicationException();
			}

		}		
		
		public List<MifareSector> Sectors
		{
			get
			{
				return sectors;	
			}
		}
		
		public bool setMaxSectors(int nbSectors)
		{
			log("Setting the new number of sectors to read ...");
			if (nbSectors <= 40)
			{
				_nbSectors = nbSectors;
				return true;
			} else
			{
				log("There can't be more than 40 sectors");
				return false;
			}
		}
		
		
		public int NbSectors
		{
			get
			{
				return _nbSectors;	
			}
			
		}
		
		
		private bool Read()
		{
			ushort address = 0;
			byte length;
			byte[] read_data;
			byte[] data_long_sectors;
			MifareSector sector;
			sectors = new List<MifareSector>();
			log("Reading Mifare Classic Card, with default keys ...");
			for(int sctr = 0; sctr < _nbSectors; sctr++)
			{
				if (address < 0x80)
				{
					length = 0x40;
					read_data = ReadBinaryWithoutKey(address, length);
					sector = new MifareSector(MF, sctr, read_data, _channel);
					sectors.Add(sector);
					address += (ushort) (length / 0x10);					
					
				} else
				{
					length = 0x80;
					read_data = ReadBinaryWithoutKey(address, length);
					if (read_data == null)
					{
						/* No access --> skip this sector	*/
						sector = new MifareSector(MF, sctr, null, _channel);
						sectors.Add(sector);
						address += (ushort) (2*length / 0x10);		
						continue;
					}

					data_long_sectors = new byte[2*read_data.Length];
					Array.ConstrainedCopy(read_data, 0, data_long_sectors, 0, read_data.Length);
					
					address += (ushort) (length / 0x10);
					
					read_data = ReadBinaryWithoutKey(address, length);
					Array.ConstrainedCopy(read_data, 0, data_long_sectors, length, read_data.Length);
					
					sector = new MifareSector(MF, sctr, data_long_sectors, _channel);
					sectors.Add(sector);
					
					address += (ushort) (length / 0x10);			
				}

			}
				
			return true;
			
		}
		
		private byte[] ReadBinaryWithoutKey(ushort address, byte length)
		{
			CAPDU capdu = new CAPDU(0xFF, 0xF3, (byte) (address / 0x0100), (byte) (address % 0x0100), length);
			
			log("< " + capdu.AsString(" ") + " (reading without specified key)");
			
			RAPDU rapdu = null;
			
		  for (int retry = 0; retry < 4; retry++)
		  {
  			rapdu = _channel.Transmit(capdu);
  			
  			if (rapdu == null)
  			  break;
  			if ((rapdu.SW != 0x6F01) && (rapdu.SW != 0x6F02) && (rapdu.SW != 0x6F0B))
  			  break;
  			
  			Thread.Sleep(10);
		  }
			
			if (rapdu == null)
			{
				log("Error '" + _channel.LastErrorAsString + "' while reading the card");
				return null;
			}		

			log("> " + rapdu.AsString(" "));
			
			if (rapdu.SW != 0x9000)
			{
				log("Bad status word " + rapdu.SWString + " while reading the card");
				return null;
			}
			
			if (!rapdu.hasData)
			{
				log("Empty response");
				return null;
			}
  			
  	  return rapdu.data.GetBytes();
  	  
		}
		
		public override bool ReadAgain()
		{
			return Read();
		}
		
	}
}
