using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Diagnostics;
using Microsoft.Win32;


namespace SpringCardApplication
{
	public class Settings
	{
		private RegistryKey Key;
		
		public string Reader;
		public bool AutoConnect;
		public bool ShowConsole;
		public bool EnableLock;
		
		public Settings()
		{
			Load();
		}

		public void Load()
		{
			try
			{
				Key = Registry.CurrentUser.CreateSubKey("SOFTWARE\\SpringCard\\" + Application.ProductName);
				
				Reader = (string) Key.GetValue("Reader", "");
				AutoConnect = ((int) Key.GetValue("AutoConnect", 1) != 0);
				ShowConsole = ((int) Key.GetValue("ShowConsole", 0) != 0);
				EnableLock = ((int) Key.GetValue("EnableLock", 0) != 0);
			}
			catch (Exception)
			{

			}
		}
		
		public void Save()
		{
	        Key.SetValue("Reader", (Reader != null) ? Reader : "");
	        Key.SetValue("AutoConnect", (int) (AutoConnect ? 1 : 0));
	        Key.SetValue("ShowConsole", (int) (ShowConsole ? 1 : 0));
	        Key.SetValue("EnableLock", (int) (EnableLock ? 1 : 0));			
		}
		
		
		public const int RememberFileCount = 5;
		public const int RememberKeyCount = 10;

		
		public string ReadString(string section, string name, string default_value)
		{
			try
			{
				return (string) Key.GetValue(section + "." + name);
			}
			catch (Exception)
			{

			}
			return default_value;
		}
		
		public string ReadString(string section, string name)
		{
			return ReadString(section, name, "");
		}

		public int ReadInteger(string section, string name, int default_value)
		{
			int r = default_value;
			string s = ReadString(section, name);
			if (s != "")
				int.TryParse(s, out r);
			return r;
		}
		
		public int ReadInteger(string section, string name)
		{
			return ReadInteger(section, name, 0);
		}
		
		public bool ReadBoolean(string section, string name, bool default_value)
		{
			return (ReadInteger(section, name, default_value ? 1 : 0) != 0);
		}

		public bool ReadBoolean(string section, string name)
		{
			return ReadBoolean(section, name, false);
		}
		
		public void WriteString(string section, string name, string value)
		{
			try
			{
				Key.SetValue(section + "." + name, value);
			}
			catch (Exception)
			{

			}
		}
		
		public void WriteInteger(string section, string name, int value)
		{
			WriteString(section, name, value.ToString());
		}
		
		public void WriteBoolean(string section, string name, bool value)
		{
			WriteInteger(section, name, value ? 1 : 0);
		}
		
		public void RememberFile(string FileName)
		{
			string[] recents = new string[RememberFileCount];
			
			for (int i=0; i<RememberFileCount; i++)
			{
				recents[i] = ReadString("recent", i.ToString());
			}
			
			int pos = RememberFileCount - 1;
			for (int i=0 ; i<RememberFileCount ; i++)
			{
				if ((recents[i]!=null) && recents[i].Equals(FileName))
					pos = i;
			}
			
			WriteString("recent", "0", FileName);
			for (int i=1; i<=pos; i++)
			{
				if (recents[i-1] != null)
					WriteString("recent", i.ToString(), recents[i-1]);
			}
		}
		
		public void RememberDirectory(string direct)
		{
			WriteString("recent", "dir", direct);
		}
		
		public string LastDirectory()
		{
			return ReadString("recent", "dir");
		}
		
		public string LastFile()
		{
			return ReadString("recent", "0");
		}
		
		public string[] LastFiles()
		{
			int i, c;
			
			for (c=0; c<RememberFileCount; c++)
			{
				string s = ReadString("recent", c.ToString());
				if ((s == null) || s.Equals(""))
					break;
			}
			
			string[] r = new string[c];
			
			for (i=0; i<c; i++)
			{
				r[i] = ReadString("recent", i.ToString());
			}

			return r;
		}
		
		public void SaveFormAspect(Form f)
		{
			string section = "form." + f.Name;
			
			if (f.WindowState == FormWindowState.Maximized)
			{
				WriteBoolean(section, "maximized", true);
			} else
			{
				WriteBoolean(section, "maximized", false);
				WriteInteger(section, "left", f.Left);
				WriteInteger(section, "top", f.Top);
				WriteInteger(section, "height", f.Height);
				WriteInteger(section, "width", f.Width);
			}
		}
		
		public void LoadFormAspect(Form f)
		{
			string section = "form." + f.Name;
			
			f.Left = ReadInteger(section, "left", f.Left);
			f.Top = ReadInteger(section, "top", f.Top);
			f.Height = ReadInteger(section, "height", f.Height);
			f.Width = ReadInteger(section, "width", f.Width);
			
			if (ReadBoolean(section, "maximized"))
				f.WindowState = FormWindowState.Maximized;
			else
				f.WindowState = FormWindowState.Normal;
		}
		
		
		public void RememberKeyA(string Key)
		{
			string[] recents = new string[RememberKeyCount];
			
			for (int i=0; i<RememberKeyCount; i++)
				recents[i] = ReadString("keyAs", i.ToString());
			
			int pos = RememberKeyCount - 1;
			for (int i=0 ; i<RememberKeyCount ; i++)
			{
				if ((recents[i]!=null) && recents[i].Equals(Key))
					pos = i;
			}
			
			WriteString("keyAs", "0", Key);
			for (int i=1; i<=pos; i++)
			{
				if (recents[i-1] != null)
					WriteString("keyAs", i.ToString(), recents[i-1]);
			}
		}

		
		public string[] LastKeyAs()
		{
			int i, c;
			
			for (c=0; c<RememberKeyCount; c++)
			{
				
				string s = ReadString("keyAs", c.ToString());
				if ((s == null) || s.Equals(""))
					break;
			}
			
			string[] r = new string[c];
			
			for (i=0; i<c; i++)
			{
				r[i] = ReadString("keyAs", i.ToString());
			}

			return r;
		}
		
		
		
		public void RememberKeyB(string Key)
		{
			string[] recents = new string[RememberKeyCount];
			
			for (int i=0; i<RememberKeyCount; i++)
			{
				recents[i] = ReadString("keyBs", i.ToString());
			}
			
			int pos = RememberKeyCount - 1;
			for (int i=0 ; i<RememberKeyCount ; i++)
			{
				if ((recents[i]!=null) && recents[i].Equals(Key))
					pos = i;
			}
			
			WriteString("keyBs", "0", Key);
			for (int i=1; i<=pos; i++)
			{
				if (recents[i-1] != null)
					WriteString("keyBs", i.ToString(), recents[i-1]);
			}
		}

		
		public string[] LastKeyBs()
		{
			int i, c;
			
			for (c=0; c<RememberKeyCount; c++)
			{
				string s = ReadString("keyBs", c.ToString());
				if ((s == null) || s.Equals(""))
					break;
			}
			
			string[] r = new string[c];
			
			for (i=0; i<c; i++)
			{
				r[i] = ReadString("keyBs", i.ToString());
			}

			return r;
		}
		
		
		
	}
}