#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "pcscsneak.h"

static const char *report_filename = NULL;
static FILE *report_file = NULL;
static BOOL report_multi_open = FALSE;
static BOOL report_multi_first = FALSE;
static _TIMESTAMP_ST report_last_timestamp = { 0 };

void report_init(const char *filename)
{

  report_filename = filename;
}

void report_open(void)
{
  if (report_filename != NULL)
    if (report_file == NULL)
      report_file = fopen(report_filename, "at");
}

void report_close(void)
{
  if (report_file != NULL)
    fclose(report_file);
  report_file = NULL;
}

void report_timestamp(void)
{
  _TIMESTAMP_ST tms;
  LARGE_INTEGER l_delta;
  DWORD delta = 0;

  get_timestamp(&tms);

  if (report_last_timestamp.current_counter.QuadPart != 0)
  { 
    l_delta.QuadPart = tms.current_counter.QuadPart - report_last_timestamp.current_counter.QuadPart;

    l_delta.QuadPart *= 1000000L;
    l_delta.QuadPart /= frequency.QuadPart;

    delta = (DWORD) l_delta.QuadPart;

    if (delta >= 60000000L) delta = 0;
  }

  memcpy(&report_last_timestamp, &tms, sizeof(_TIMESTAMP_ST));

  printf("%02d:%02d:%02d.%03d ", tms.current_time.wHour, tms.current_time.wMinute, tms.current_time.wSecond, tms.current_time.wMilliseconds);
  if (delta)
    printf("%02ld.%06ld", delta / 1000000L, delta % 1000000L);
  else
    printf("         ");

  if (report_file != NULL)
  {
    fprintf(report_file, "%02d/%02d/%02d ", tms.current_time.wDay, tms.current_time.wMonth, tms.current_time.wYear);
    fprintf(report_file, "%02d:%02d:%02d.%03d ", tms.current_time.wHour, tms.current_time.wMinute, tms.current_time.wSecond, tms.current_time.wMilliseconds);
    if (delta)
      fprintf(report_file, "%02ld.%06ld", delta / 1000000L, delta % 1000000L);
    else
      fprintf(report_file, "         ");
  }
}

void report_timestamp_empty(void)
{
  printf("                      ");
  if (report_file != NULL)
    fprintf(report_file, "                                 ");
}

void report_print(const char *fmt, ...)
{
  va_list arg_ptr;

  va_start(arg_ptr, fmt);
  vfprintf(stdout, fmt, arg_ptr);
  va_end(arg_ptr);

  if (report_file != NULL)
  {
    va_start(arg_ptr, fmt);
    vfprintf(report_file, fmt, arg_ptr);
    va_end(arg_ptr);
  }
}

void report_single(const char *fmt, ...)
{
  va_list arg_ptr;

  report_open();
  report_timestamp();
  report_print(" ");

  va_start(arg_ptr, fmt);
  vfprintf(stdout, fmt, arg_ptr);
  va_end(arg_ptr);

  if (report_file != NULL)
  {
    va_start(arg_ptr, fmt);
    vfprintf(report_file, fmt, arg_ptr);
    va_end(arg_ptr);
  }

  report_print("\n");
  report_close();
}

void report_multi_begin(void)
{
  if (report_multi_open)
    report_multi_end();

  report_open();

  report_multi_open = TRUE;
  report_multi_first = TRUE;
}

void report_multi_add(const char *fmt, ...)
{
  va_list arg_ptr;

  if (!report_multi_open) return;

  if (report_multi_first)
    report_timestamp();
  else
    report_timestamp_empty();

  report_print(" ");

  va_start(arg_ptr, fmt);
  vfprintf(stdout, fmt, arg_ptr);
  va_end(arg_ptr);

  if (report_file != NULL)
  {
    va_start(arg_ptr, fmt);
    vfprintf(report_file, fmt, arg_ptr);
    va_end(arg_ptr);
  }
  
  report_print("\n");

  report_multi_first = FALSE;
}

void report_multi_add_buffer(const char *str, const unsigned char *buffer, size_t length)
{
  size_t i;

  if (!report_multi_open) return;

  if (report_multi_first)
    report_timestamp();
  else
    report_timestamp_empty();

  report_print(" ");

  if (str != NULL)
    report_print(str);

  for (i=0; i<length; i++)
    report_print("%02X", buffer[i]);
  
  report_print("\n");

  report_multi_first = FALSE;
}

void report_multi_end(void)
{
  if (!report_multi_open) return;

  if (report_multi_first)
  {
    report_timestamp();
    report_print("  -- Nothing!\n");
  }

  report_multi_open = FALSE;
  report_close();
}


