#include "quickbatch.h"

BOOL smartcard_get_uid(smartcard_t card, BYTE uid[], BYTE *uidlen)
{
  unsigned char capdu[5];
	unsigned char rapdu[256+2];
	unsigned long rapdu_length;

	if ((card == NULL) || (uid == NULL) || (uidlen == NULL))
	  return FALSE;

	capdu[0] = 0xFF;
	capdu[1] = 0xCA;
	capdu[2] = 0x00;
	capdu[3] = 0x00;
	capdu[4] = 0x00;


	rapdu_length = sizeof(rapdu);
	if (!pcsc_transmit(card, capdu, sizeof(capdu), rapdu, &rapdu_length))
		return FALSE;

	if ((rapdu[rapdu_length-2] != 0x90) || (rapdu[rapdu_length-1] != 0x00))
	{
	  if (be_verbose)
		  printf("Get UID failed (SW=%02X%02X)\n", rapdu[rapdu_length-2], rapdu[rapdu_length-1]);
		return FALSE;
	}

  if (rapdu_length-2 > *uidlen)
  {
    if (be_verbose)
      printf("Card UID is longer than our buffer\n");
    return FALSE;
  }

  *uidlen = (BYTE) (rapdu_length-2);
  memcpy(uid, rapdu, *uidlen);

  return TRUE;
}

BOOL smartcard_read_memory(smartcard_t card, unsigned short address, unsigned char buffer[], unsigned char length)
{
  unsigned char capdu[5];
	unsigned char rapdu[256+2];
	unsigned long rapdu_length;

	if ((card == NULL) || (buffer == NULL))
	  return FALSE;

	capdu[0] = 0xFF;
	capdu[1] = 0xB0;
	capdu[2] = (unsigned char) (address >> 8);
	capdu[3] = (unsigned char) (address);
	capdu[4] = length;


	rapdu_length = sizeof(rapdu);
	if (!pcsc_transmit(card, capdu, sizeof(capdu), rapdu, &rapdu_length))
		return FALSE;

	if ((rapdu[rapdu_length-2] != 0x90) || (rapdu[rapdu_length-1] != 0x00))
	{
	  if (be_verbose)
		  printf("Card read error at address %d, SW=%02X%02X\n", address, rapdu[rapdu_length-2], rapdu[rapdu_length-1]);
		return FALSE;
	}

  if (length == 0)
  {
    length = (BYTE) (rapdu_length-2);
  } else
  if (rapdu_length-2 < length)
	{
	  if (be_verbose)
		  printf("Card read failed at address %u (length=%lu<%u)\n", address, rapdu_length-2, length);
    return FALSE;
	}

	if (buffer != NULL)
	  memcpy(buffer, rapdu, length);

  return TRUE;
}

