#include "quickbatch.h"

BOOL springcard_suspend_tracking(smartcard_t card)
{
  BYTE capdu[] = { 0xFF, 0xFB, 0x01, 0x00 };
	unsigned char rapdu[256+2];
	unsigned long rapdu_length;

	rapdu_length = sizeof(rapdu);
	if (!pcsc_transmit(card, capdu, sizeof(capdu), rapdu, &rapdu_length))
		return FALSE;

	if ((rapdu[rapdu_length-2] != 0x90) || (rapdu[rapdu_length-1] != 0x00))
	{
	  if (be_verbose)
		  printf("Suspend tracking failed (SW=%02X%02X)\n", rapdu[rapdu_length-2], rapdu[rapdu_length-1]);
		return FALSE;
	}

  return TRUE;
}

BOOL springcard_resume_tracking(smartcard_t card)
{
  BYTE capdu[] = { 0xFF, 0xFB, 0x00, 0x00 };
	unsigned char rapdu[256+2];
	unsigned long rapdu_length;

	rapdu_length = sizeof(rapdu);
	if (!pcsc_transmit(card, capdu, sizeof(capdu), rapdu, &rapdu_length))
		return FALSE;

	if ((rapdu[rapdu_length-2] != 0x90) || (rapdu[rapdu_length-1] != 0x00))
	{
	  if (be_verbose)
		  printf("Resume tracking failed (SW=%02X%02X)\n", rapdu[rapdu_length-2], rapdu[rapdu_length-1]);
		return FALSE;
	}

  return TRUE;
}

BOOL springcard_wait_card_present(smartcard_t card)
{
  BYTE capdu[] = { 0xFF, 0xFB, 0x01, 0x02 };
	unsigned char rapdu[256+2];
	unsigned long rapdu_length;

  for (;;)
  {
	  rapdu_length = sizeof(rapdu);
	  if (!pcsc_transmit(card, capdu, sizeof(capdu), rapdu, &rapdu_length))
		  return FALSE;

	  if (rapdu[rapdu_length-2] != 0x90)
	  {
	    if (be_verbose)
		    printf("Wait card present failed (SW=%02X%02X)\n", rapdu[rapdu_length-2], rapdu[rapdu_length-1]);
		  return FALSE;
	  }

    switch (rapdu[rapdu_length-1])
    {
      case 0x02 : return TRUE; /* Card present, no collision */
      default   : break;       /* Card absent or card present but with collision */
    }
  }
}

BOOL springcard_wait_card_absent(smartcard_t card)
{
  BYTE capdu[] = { 0xFF, 0xFB, 0x01, 0x01 };
	unsigned char rapdu[256+2];
	unsigned long rapdu_length;

  for (;;)
  {
	  rapdu_length = sizeof(rapdu);
	  if (!pcsc_transmit(card, capdu, sizeof(capdu), rapdu, &rapdu_length))
		  return FALSE;

	  if (rapdu[rapdu_length-2] != 0x90)
	  {
	    if (be_verbose)
		    printf("Wait card absent failed (SW=%02X%02X)\n", rapdu[rapdu_length-2], rapdu[rapdu_length-1]);
		  return FALSE;
	  }

    switch (rapdu[rapdu_length-1])
    {
      case 0x01 : return TRUE; /* Card absent */
      default   : break;       /* Card still present */
    }
  }
}


